"""
gen dot diagram for inspiration
"""
import random

def randstr(length, charset):
    """Generate a random string of given length from the given character set."""
    return "".join(random.choice(charset) for i in range(length))

# Set up node and edge data
NUM_NODES = 6
nodes = [f"boat{i}" for i in range(1, NUM_NODES+1)]
edges = []
for i in range(NUM_NODES):
    num_edges = random.randint(1, 5)
    for j in range(num_edges):
        k = random.randint(0, NUM_NODES-1)
        if k != i and (i, k) not in edges and (k, i) not in edges:
            edges.append((i, k))

# Generate dot file
with open("signal_k_observation.dot", "w", encoding="utf-8") as f:
    f.write("graph signal_k_observation {\n")
    f.write("    node [style=filled, color=white, fontname=Arial, fontsize=16]\n")
    f.write("    edge [color=gray, penwidth=2, style=dashed]\n")
    f.write("    \n")
    for i, node in enumerate(nodes):
        f.write(f'    {node} [shape=circle, fontsize=16]\n')
        for j in range(4):
            LABEL = randstr(4, "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789")
            color = ["#8fb1cc", "#9bd3b4", "#f6b0b7", "#c6c6c6"][j]
            f.write(f'    {node}_{j} [label="{LABEL}", shape=rectangle,  \
            style=filled, color="{color}", fontcolor=white, width=0.7,  \
            height=0.7, fixedsize=true]\n')
            f.write(f'    {node} -- {node}_{j} [style=invis]\n')
    f.write("    \n")
    for edge in edges:
        f.write(f"    {nodes[edge[0]]} -- {nodes[edge[1]]} [penwidth=2, style=dashed]\n")
    f.write("}\n")



#!/usr/bin/env python3
"""
gen signal k json for testing the navactor graph features
"""
import random
import math
import uuid
from datetime import datetime

country_codes = {
    "200": range(0, 20),  # Test MID range for illustration purposes
    "201": range(0, 20),  # Test MID range for illustration purposes
    "202": range(0, 20),  # Test MID range for illustration purposes
    "203": range(0, 20),  # Test MID range for illustration purposes
    "204": range(0, 20),  # Test MID range for illustration purposes
    "205": range(0, 20)   # Test MID range for illustration purposes
}

def generate(num_boats, base_coords, nautical_miles):
    """
    gen signal k json
    """
    vessels = {}
    for i in range(1, num_boats + 1):
        # Generate MMSI based on country code, MID, and unique vessel ID
        country_code = random.choice(list(country_codes.keys()))
        mid_range = country_codes[country_code]
        mid = random.choice(mid_range)
        vessel_id = random.randint(1000, 9999)
        mmsi = f"{country_code}{mid}{vessel_id}"

        boat_data = {
            "name": f"Boat {i}",
            "uuid": f"urn:mrn:signalk:uuid:{str(uuid.uuid4())}",
            "mmsi": mmsi,
            "navigation": {
                "position": {
                    "value": {
                        "latitude": base_coords[0] + (random.random() * 2 - 1) *
                        (nautical_miles / 60),
                        "longitude": base_coords[1] + (random.random() * 2 - 1) *
                        (nautical_miles / 60) / math.cos(base_coords[0] *
                                                         math.pi / 180),
                        "altitude": 0.0
                    },
                    "$source": "self",
                    "timestamp": datetime
                    .utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ')
                }
            },
            "headingMagnetic": {
                "value": 0.0,
                "$source": "self",
                "timestamp": datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
            },
            "speedOverGround": {
                "value": 0.0,
                "$source": "self",
                "timestamp": datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
            },
            "courseOverGroundTrue": {
                "value": 0.0,
                "$source": "self",
                "timestamp": datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
            }
        }
        vessels[f"urn:mrn:signalk:uuid:{boat_data['uuid']}"] = boat_data
    return {"vessels": vessels}



#!/usr/bin/env python3
"""
gen signal k json for testing the navactor graph features
"""
import argparse
import json
import uuid
from signalkgen.move_boats import move_boats
from signalkgen.generate import generate

def main():
    """
    entry point
    """
    parser = argparse.ArgumentParser(description='Generate and move boats in Signal K format')
    parser.add_argument('--num-boats', type=int, default=5, help='Number of boats to generate')
    parser.add_argument('--latitude', type=float, default=37.7749,
                        help='Base latitude for generating boats')
    parser.add_argument('--longitude', type=float, default=-122.4194,
                        help='Base longitude for generating boats')
    parser.add_argument('--nautical-miles', type=float, default=10,
                        help='Range in nautical miles for generating boats')
    parser.add_argument('--iterations', type=int, default=3,
                        help='Number of iterations to move boats')
    args = parser.parse_args()

    # Generate initial boat positions
    data = generate(args.num_boats, (args.latitude,
                    args.longitude), args.nautical_miles)
    signal_k_data = {
        "version": "1.0.0",
        "self": f"urn:mrn:signalk:uuid:{str(uuid.uuid4())}",
        "vessels": data["vessels"],
        "sources": {
            "self": {
                "type": "internal",
                "src": "signalkgen"
            }
        }
    }
    boat_data = [signal_k_data]

    # Move boats and print new positions
    for _ in range(args.iterations):
        data = move_boats(data)
        signal_k_data = {
            "version": "1.0.0",
            "self": f"urn:mrn:signalk:uuid:{str(uuid.uuid4())}",
            "vessels": data["vessels"],
            "sources": {
                "self": {
                    "type": "internal",
                    "src": "signalkgen"
                }
            }
        }
        boat_data.append(signal_k_data)

    # Convert list of dictionaries to JSON string
    print(json.dumps(boat_data, indent=2))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
gen signal k json for testing the navactor graph features
"""
import random
import math
from datetime import datetime


def move_boats(signal_k_data):
    """
    boats should move along a vector
    """
    vessels = signal_k_data["vessels"]
    for vessel_id in vessels:
        boat = vessels[vessel_id]
        # calculate new position based on current position, course over ground
        # true, and speed over ground
        lat = boat["navigation"]["position"]["value"]["latitude"]
        lon = boat["navigation"]["position"]["value"]["longitude"]
        cog_true = boat["courseOverGroundTrue"]["value"]
        sog = boat["speedOverGround"]["value"]
        lat += (sog / 60) * math.cos(cog_true * math.pi / 180)
        lon += (sog / 60) * math.sin(cog_true *
                                     math.pi / 180) / math.cos(lat *
                                                               math.pi / 180)
        # update boat position, heading, and speed
        boat["navigation"]["position"]["value"]["latitude"] = lat
        boat["navigation"]["position"]["value"]["longitude"] = lon
        boat["headingMagnetic"]["value"] = random.uniform(0, 360)
        boat["speedOverGround"]["value"] = random.randint(1, 15)
        boat["courseOverGroundTrue"]["value"] = boat["headingMagnetic"]["value"]
        boat["navigation"]["position"]["timestamp"] = datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
        boat["headingMagnetic"]["timestamp"] = datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
        boat["speedOverGround"]["timestamp"] = datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
        boat["courseOverGroundTrue"]["timestamp"] = datetime.utcnow().strftime(
                    '%Y-%m-%dT%H:%M:%S.%fZ')
    return {"vessels": vessels}
